function convert(input) {
  const output = input.replace(
    /^\{(.*?)}\n~~~~~~~\n(.*?)~~~~~~~\n?/gms,
    (match, tag, codeblock) => {
      const titleMatch = tag.match('title="(.*?)"');
      const title = titleMatch ? titleMatch[1] : '';

      const languageMatch = tag.match('lang="(.*?)"');
      const language = languageMatch ? languageMatch[1] : '';

      const globalOffset = title === '' ? 0 : 1;
      const ranges = buildRanges(match.split('\n'), globalOffset)
        .map(formatRange)
        .join(',');

      const header = '```' + language + (ranges ? `{${ranges}}` : '');
      let block = removeLeanpubComments(codeblock);
      const footer = '```\n';

      return [header, addComment(title, block.trim()), footer].join('\n');
    }
  );

  return removeLeanpubComments(output);
}

function stripLines(text, needles) {
  return needles.reduce(
    (output, needle) => output.split(needle + '\n').join(''),
    text
  );
}

// Assumes the "leanpub" comments are well-formatted and have pairs
function buildRanges(codeLines, globalOffset = 0) {
  const ranges = [];

  let headerOffset = 0;
  let previousStart = null;

  codeLines.forEach((currentLine, index) => {
    if (currentLine.indexOf('# leanpub-start-insert') !== -1) {
      previousStart = index + 1;
      headerOffset += 1;
      return;
    }

    if (currentLine.indexOf('# leanpub-end-insert') !== -1) {
      headerOffset += 1;
      ranges.push([
        previousStart - headerOffset + globalOffset,
        index - 1 - headerOffset + globalOffset
      ]);
    }
  });

  return ranges;
}

function formatRange([from, to]) {
  return from === to ? from : `${from}-${to}`;
}

function removeLeanpubComments(text) {
  return stripLines(text, ['# leanpub-start-insert', '# leanpub-end-insert']);
}

function addComment(comment, block) {
  return comment === '' ? block : `// ${comment}\n` + block;
}

module.exports = { convert };
